@@ -18,17 +18,8 @@ use warnings;
 
 require Exporter;
 
-our $VERSION = '0.04';
-## 0.04 Renamed BerkeleyDB::Lite
-##	Added %dbreg registry
-## 0.03 Added uniquepairs
-##	Added subclass property
-## 	Added uniquekeys
-## 	Added recover option to envsetup
-## 0.02 Added BerkeleyDB::Lite::Btree::Lexical
-##	Added BerkeleyDB::Lite::Btree lexical constructor
-##      Added rootdir property
-##      changed filter_store_value in new
+our $VERSION = '0.10';
+## See Changes file
 
 our @defaults = ( 
 		rootdir => "/usr/local/apache/cgi-bin/db" 
@@ -56,7 +47,14 @@ our @EXPORT = qw( duplicatekeys incrementkeys uniquepairs uniquekeys );
 ## closed before database objects.  %dbreg closes all open databases 
 ## first.
 END {
+	closeall() ;
+	}
+
+## Run closeall if forking child processes.  All references must be destroyed
+## first.
+sub closeall {
 	map { eval { $_->db_close } } values %dbreg ;
+	%env = () ;
 	}
 
 sub new {
@@ -202,10 +200,8 @@ sub scalars {
 	my( $self, %self ) ;
 
 	my %env = BerkeleyDB::Lite::envsetup( @_ ) ;
-	croak $! unless $env{'-Env'} ;
 
 	my %alt = @_ ;
-
 	my $filename = $env{filename} ;
 	delete $env{filename} ;
 
@@ -213,7 +209,8 @@ sub scalars {
 	$self = tie %self, $alt{subclass} || 'BerkeleyDB::Hash',
 			%env,
 			-Flags => DB_CREATE, 
-			or warn "($$) $filename: $!" ;
+			or warn "($$) $filename: $!"
+			if $env{'-Env'} ;
 
 	## Failsafe- open R/O without locking
 	if ( $filename && ! $self ) {
@@ -309,9 +306,7 @@ sub dosearch {
 			$unique{ $key }++ ;
 			}
 		else {
-			push @keys, $key ;
-			push @values, $value ;
-			push @each, $key, $value ;
+			push @each, [ $key, $value ] ;
 			}
 
 		$status = $cursor->c_get( $key, $value, DB_NEXT ) ;
@@ -319,23 +314,20 @@ sub dosearch {
 
 	
 	$cursor = undef ;
-	return [ keys %unique ], [], [ %unique ] if $isunique ;
-	return [ @keys ], [ @values ], [ @each ] ;
+	@each = map { [ $_, $unique{$_} ] } keys %unique if $isunique ;
+	return \@each ;
 	}
 
 sub matchingkeys {
-	my @s = dosearch( @_ ) ;
-	return @{ $s[0] } ;
+	return map { $_->[0] } @{ dosearch( @_ ) } ;
 	}
 
 sub matchingvalues {
-	my @s = dosearch( @_ ) ;
-	return @{ $s[1] } ;
+	return map { $_->[1] } @{ dosearch( @_ ) } ;
 	}
 
 sub searchset {
-	my @s = dosearch( @_ ) ;
-	return @{ $s[2] } ;
+	return map { @$_ } @{ dosearch( @_ ) } ;
 	}
 
 ## intended for Btree's with incremented keys
@@ -473,7 +465,7 @@ methods for returning multiple record sets.
 =head2 Example 1
 
 BerkeleyDB::Lite maintains BerkeleyDB environment references
-in a package scoped hash keyed on the B<home> argument.  The 
+in a package variable hash keyed on the B<home> argument.  The 
 basic BerkeleyDB::Lite constructor arguments define the 
 BerkeleyDB environment and database.  When the constructor 
 is called, a previously opened environment is used if 
@@ -488,7 +480,7 @@ By default, BerkeleyDB::Lite is designed to marshall objects into a
 database using the B<Storable> module.
 
 Example 1 shows a simple application that illustrates both of 
-these features.  The constructor call contains the minimum 
+these features.  The constructor is called with the minimum 
 arguments to identify the environment and the database.
 
 These few lines of code are sufficient to add persistent object 
@@ -517,8 +509,8 @@ BerkeleyDB::Lite::Hash classes.
 
 The B<delete> method is used to delete an element from the list.  
 Since BerkeleyDB::Lite adheres to the B<Tie> interface, the 
-B<delete> keyword can normally used to remove stored objects.  
-The B<delete> method should be used on databases with duplicate 
+B<delete function> can normally used to remove stored objects.  
+The B<delete method> should be used on databases with duplicate 
 keys to avoid indeterminate results.
 
 BerkeleyDB returns the status of a delete operation.  This 
@@ -529,12 +521,10 @@ idiom:
 
 
 A BerkeleyDB database configured for duplicate keys also allows 
-duplicate key/value pairs.  For most one-to-many data sets, key 
-value pairs should be unique.  There are several ways to handle 
-this issue, but none of them are currently implemented. 
-
-Commonly, the workaround is to import a retrieved list into a 
-hash structure:
+duplicate key/value pairs.  For most one-to-many data sets, 
+key/value pairs should be unique.  This issue has not been 
+completely resolved.  Presently, the workaround is to import a 
+retrieved list into a hash structure:
 
   %unique = map { $_ => 1 } $types->recordset('primate') ;
   keys %unique ;
@@ -559,7 +549,7 @@ difficulty:
 		&duplicatekeys,
 		) ;
 
-  my $Felix = new BigCat( dinner => 'antelope' ) ;
+  my $Felix = new BigCat dinner => 'antelope' ;
   $cats->{lion} = $Felix ;
   $Felix->{dinner} = 'gazelle' ;
   $cats->delete( lion => $Felix ) ;		## fails
@@ -575,8 +565,8 @@ context.  The following example illustrates the situation:
   $weight =~ s/\D//g ;
   my $Felix = new BigCat( weight => $weight ) ;	## member as string
   $cats->{lion} = $Felix ;
-  $cats->delete( lion => $Felix )		## member as integer 
-		if $Felix->{weight} > 200 ;	## fails
+  $cats->delete( lion => $Felix )		## operation fails
+		if $Felix->{weight} > 200 ;	## member as integer 
 
 
 =head2 Example 3
